package cn.bunny.security.handelr;

import cn.bunny.common.result.Result;
import cn.bunny.enums.ResultCodeEnum;
import com.alibaba.fastjson2.JSON;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;

import java.io.IOException;

/**
 * 请求未认证接口
 */
@Slf4j
public class SecurityAuthenticationEntryPoint implements AuthenticationEntryPoint {
    @Override
    public void commence(HttpServletRequest request, HttpServletResponse response, AuthenticationException authException) throws IOException, ServletException {
        String token = response.getHeader("token");
        String message = authException.getLocalizedMessage();

        log.info("请求未认证接口：{}，用户id：{}", message, token);

        // 创建结果对象
        Result<Object> result = Result.error(ResultCodeEnum.FAIL_REQUEST_NOT_AUTH);

        // 返回响应
        response.setContentType("application/json;charset=UTF-8");
        response.getWriter().println(JSON.toJSON(result));
    }
}
